@@ -52,10 +52,10 @@ CHARACTERISTIC_UUID_TO_SERVER = "e7a60001-6639-429f-94fd-86de8ea26897"
 CHARACTERISTIC_UUID_FROM_SERVER = "e7a60002-6639-429f-94fd-86de8ea26897"
 
 # Enable to debug framing and deframing of commands
-DEBUG_COMMAND_FRAMING = False
+DEBUG_COMMAND_FRAMING = True
 
 # Enable to debug decrypt/encrypt of commands
-DEBUG_COMMAND_CRYPT = False
+DEBUG_COMMAND_CRYPT = True
 
 DISCONNECT_DELAY = 30
 
@@ -145,7 +145,7 @@ class AssociateProcedure(BaseProcedure):
 
         ack_fut = self._lock.receive_once(cmds.AckRsp)
         auth_check_fut = self._lock.receive_once(cmds.AuthCheckCmd)
-        self._lock.set_crypto(crypto, 1)
+        self._lock.set_crypto(crypto, 2)
         await self._lock.send_cmd(cmds.AuthPinChallengeReplyCmd(reply))
         await ack_fut
         if ack_fut.result().error_code != ErrorCode.SUCCESS:
@@ -238,13 +238,12 @@ class AuthenticateProcedure(BaseProcedure):
         await asyncio.gather(ident_cmd_fut, ident_rsp_fut)
         self._lock.on_identification(ident_rsp_fut.result())
 
+        auth_challenge_fut = self._lock.receive_once(cmds.AuthChallengeCmd)
+        not_associated_fut = self._lock.receive_once(cmds.NotAssociatedCmd)
         await self._lock.send_cmd(
             cmds.GetIdentificationRsp.defaults(self._key_holder_id)
         )
 
-        auth_challenge_fut = self._lock.receive_once(cmds.AuthChallengeCmd)
-        not_associated_fut = self._lock.receive_once(cmds.NotAssociatedCmd)
-
         done, pending = await asyncio.wait(
             (auth_challenge_fut, not_associated_fut),
             return_when=asyncio.FIRST_COMPLETED,
@@ -262,10 +261,9 @@ class AuthenticateProcedure(BaseProcedure):
         auth_challenge_accepted_fut = self._lock.receive_once(
             cmds.AuthChallengeAcceptedCmd
         )
+        self._lock.set_crypto(crypto, 1)
         await self._lock.send_cmd(cmds.AuthChallengeRsp(reply))
 
-        self._lock.set_crypto(crypto, 0)
-
         await auth_challenge_accepted_fut
         _LOGGER.debug("%s: Challenge accepted", self._lock.name)
         await self._lock.send_cmd(
@@ -396,6 +394,17 @@ class UnlockProcedure(BaseProcedure):
         return True
 
 
+class NullProcedure(BaseProcedure):
+    """Do nothing."""
+
+    enable_notifications = True
+    need_auth = True
+
+    async def execute(self) -> bool:
+        """Execute the procedure"""
+        return True
+
+
 class DKEYLock:
     """Manage a Dormakaba DKEY lock."""
 
@@ -496,7 +505,7 @@ class DKEYLock:
         self._secret = association_data.secret
 
     @staticmethod
-    def _validate_activate_code(activation_code: str) -> str:
+    def _validate_activation_code(activation_code: str) -> str:
         """Validate the activation code, raises if it's not valid."""
         trans = str.maketrans("", "", "- \t")
         activation_code = activation_code.translate(trans)
@@ -508,7 +517,7 @@ class DKEYLock:
     async def associate(self, activation_code: str) -> AssociationData:
         """Associate with the lock."""
         _LOGGER.debug("%s: Associate %s", self.name, activation_code)
-        activation_code = self._validate_activate_code(activation_code)
+        activation_code = self._validate_activation_code(activation_code)
         associate_proc = AssociateProcedure(self, activation_code)
         if (
             not await self._execute(associate_proc)
@@ -571,10 +580,11 @@ class DKEYLock:
         unlock_proc = UnlockProcedure(self)
         return await self._execute(unlock_proc)
 
-    async def update(self) -> None:
+    async def update(self) -> bool:
         """Update the lock's status."""
         _LOGGER.debug("%s: Update", self.name)
-        await self._enable_notifications()
+        null_proc = NullProcedure(self)
+        return await self._execute(null_proc)
 
     @retry_bluetooth_connection_error(DEFAULT_ATTEMPTS)  # type: ignore[misc]
     async def _execute(self, procedure: BaseProcedure) -> bool:
@@ -604,18 +614,6 @@ class DKEYLock:
                 self._disconnect(DisconnectReason.ERROR)
                 raise
 
-    async def _enable_notifications(self) -> None:
-        """Enable notifications."""
-        if self._procedure_lock.locked():
-            _LOGGER.debug(
-                "%s: Procedure already in progress, waiting for it to complete; "
-                "RSSI: %s",
-                self.name,
-                self.rssi,
-            )
-        async with self._procedure_lock:
-            await self._enable_notifications_when_locked()
-
     async def _enable_notifications_when_locked(self) -> None:
         """Enable notifications."""
         await self._ensure_authenticated()
@@ -741,6 +739,14 @@ class DKEYLock:
 
     async def _execute_disconnect(self, reason: DisconnectReason) -> None:
         """Execute disconnection."""
+        _LOGGER.debug("%s: Execute disconnect", self.name)
+        if self._connect_lock.locked():
+            _LOGGER.debug(
+                "%s: Connection already in progress, waiting for it to complete; "
+                "RSSI: %s",
+                self.name,
+                self.rssi,
+            )
         async with self._connect_lock:
             client = self._client
             self._client = None
@@ -750,6 +756,7 @@ class DKEYLock:
                 await client.stop_notify(CHARACTERISTIC_UUID_FROM_SERVER)
                 await client.disconnect()
             self._reset(reason)
+        _LOGGER.debug("%s: Execute disconnect done", self.name)
 
     def _reset(self, reason: DisconnectReason) -> None:
         """Reset."""
@@ -833,7 +840,9 @@ class DKEYLock:
 
         if self._rx_segment:
             if DEBUG_COMMAND_FRAMING:
-                _LOGGER.info("RX: segment: %02x: %s", characteristic.handle, data.hex())
+                _LOGGER.debug(
+                    "RX: segment: %02x: %s", characteristic.handle, data.hex()
+                )
             if self._rx_segment_cmd_id != data[0]:
                 _LOGGER.warning(
                     "RX: got %s, expected %s", data[0] & 0x7F, self._rx_segment_cmd_id
@@ -899,6 +908,9 @@ class DKEYLock:
 
         self._fragmentize(char_specifier, data[0], data[2:])
 
+        crypto_enabled = self._crypto and not self._crypto_delay
+        if self._crypto and self._crypto_delay:
+            self._crypto_delay -= 1
         while self._tx_queue:
             dest, frag = self._tx_queue.popleft()
             if DEBUG_COMMAND_FRAMING:
@@ -906,7 +918,7 @@ class DKEYLock:
             got_fragment_ack: asyncio.Future[cmds.AckRsp] | None = None
             if frag[0] & 0x80:
                 got_fragment_ack = self.receive_once(cmds.AckRsp)
-            if self._crypto and not self._crypto_delay:
+            if self._crypto and crypto_enabled:
                 frag = self._crypto.encrypt_send(frag)
                 if DEBUG_COMMAND_CRYPT:
                     _LOGGER.debug("TX ct: %s: %s", dest, frag.hex())
